#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jan 22 11:07:33 2024

@author: jasonball
"""

### standard imports ###
import numpy as np
import matplotlib.pyplot as plt
import time

### import MiM plus required instruments ###
from moku.instruments import MultiInstrument
from moku.instruments import LockInAmp
from moku.instruments import WaveformGenerator
from moku.instruments import Oscilloscope

### Establish connection and number of slots being used ###
mim = MultiInstrument('10.0.0.245', force_connect=True, platform_id=4)
idn = mim.serial_number()
prop = mim.describe()
print('Connected to ' + prop['hardware'] + ', SN ' + idn)
print('Firmware version ' + str(prop['firmware']))

### Define instrument in each slot ###
wg = mim.set_instrument(1, WaveformGenerator)
lia = mim.set_instrument(2, LockInAmp)
vco = mim.set_instrument(3, WaveformGenerator)
osc = mim.set_instrument(4, Oscilloscope)

print('Instruments Set')

### Define connection map ###

connections = [dict(source="Slot1OutA", destination="Slot2InA"),
               dict(source="Slot1OutA", destination="Slot4InA"),
               dict(source="Slot2OutA", destination="Slot3InA"),
               dict(source="Slot3OutA", destination="Slot2InB"),
               dict(source="Slot3OutA", destination="Slot4InB"),
               ]

mim.set_connections(connections=connections)

print('Connections Set')

### Set up Frequency Source

wg.generate_waveform(channel=1, type='Sine', amplitude=1, frequency=100.0e6)

### Set up VCO ###

vco.generate_waveform(channel=1, type='Sine', amplitude=1, frequency=100.05e6)
vco.set_modulation(channel=1, type='Frequency', source='InputA',depth=50e3)

print('VCO configured')
### Set demodulation frequency on LIA ###

lia.set_demodulation(mode='ExternalPLL')
lia.set_pll(auto_acquire=True, bandwidth='1MHz')
lia.set_filter(corner_frequency=200e3,slope="Slope6dB")
lia.set_polar_mode("2Vpp")
lia.set_outputs(main="Theta", main_offset=0, aux='Aux', aux_offset=0)
lia.set_gain(main=10, aux=0, main_invert=False)

print('Lock-in configured')

### set the timebase for the oscilloscope ### 
osc.set_timebase(-25e-9, 25e-9)

print('oscillator configured')

### Let's plot ### 

plt.ion()
fig = plt.figure()
ax = fig.add_subplot(111)
ax.set_xlabel('Time (s)')
ax.set_ylabel('Voltage (V)')
data = osc.get_data()
line1, = ax.plot(data['time'], data['ch1'], 'r', label='ChA')
line1.set_label('ChA') 
line2, = ax.plot(data['time'], data['ch2'], 'b', label='ChB')
line2.set_label('ChB')  
ax.legend(loc='upper right')

for x in range(0,500):
    data = osc.get_data()
    line1.set_ydata(data['ch1'])
    line2.set_ydata(data['ch2'])
    fig.canvas.draw()
    fig.canvas.flush_events()
    time.sleep(0.1)

mim.relinquish_ownership()